#!/bin/bash
# Copyright 2008 Michael Pagano mpagano@gentoo.org
# Distributed under the terms of the GNU General Public License v2

version=0.1

mailCommand="/usr/sbin/sendmail"
mailTo="<change this!>"
mailFrom="<change this, also>"
subject="Alert from kernel-alert process"
dataFile="kernel-alert.data"

kernelURL="http://www.kernel.org/"
kernelFile="index.html"
tempFile="index.out"

usage() {
	echo "kernel-alert.sh [-h | -v]"
	echo "where"
	echo "	-v - print version information"
	echo "	-h - print usage"
	exit
}

for arg in $* ; do
    case "${arg}" in
    -v)
        echo "kernel-alert.sh version: $version"
		exit
        ;;
    -h)
        usage
		exit
        ;;
    esac
done

stable_flag=0
prepatch_flag=0
snapshot_flag=0
twofour_flag=0

stable=""
prepatch=""
snapshot=""
twofour=""

first_run_flag=0

message="The following kernel versions have been updated:"
alert=false

parseLine() {
	
	line="$@"

	# check for any flags set, this means the next line contains the data
	if [[ $stable_flag == 1 ]]; then
		stable=`expr x"$line" : '.*bz2>\([^]]*\)</a'`
		if [[ $stable != "" ]]; then
			stable_flag=0
		fi
	elif [[ $prepatch_flag == 1 ]]; then
		prepatch=`expr x"$line" : '.*bz2>\([^]]*\)</a'`
		if [[ $prepatch != "" ]]; then
			prepatch_flag=0
		fi
	elif [[ $snapshot_flag == 1 ]]; then
		snapshot=`expr x"$line" : '.*bz2>\([^]]*\)</a'`
		if [[ $snapshot != "" ]]; then
			snapshot_flag=0
		fi
	elif [[ $twofour_flag == 1 ]]; then
		twofour=`expr x"$line" : '.*bz2>\([^]]*\)</a'`
		if [[ $twofour != "" ]]; then
			twofour_flag=0
		fi
	fi

	# search for stable pre line
	match="stable version of the Linux kernel"
	if [[ "$line" =~ "${match}" ]]; then 
		stable_flag=1
	fi

	# search for prepatch line
	match="prepatch</a> for the stable Linux kernel tree is"
	if [[ "$line" =~ "${match}" ]]; then 
		prepatch_flag=1
	fi

	# search for snapshot line
	match="snapshot</a> for the stable Linux kernel tree is"
	if [[ "$line" =~ "${match}" ]]; then 
		snapshot_flag=1
	fi
	#  search for two_four_line
	match="The latest 2.4 version"
	if [[ "$line" =~ "${match}" ]]; then 
		twofour_flag=1
	fi
}


loadKernelAlertDataFile() {

	orig_stable=""
	orig_snapshot=""
	orig_prepatch=""
	orig_twofour=""

	if [ ! -e $dataFile ]; then
		# Datafile ($dataFile) does not exist. File will be created
		eval `touch ${dataFile}`
		first_run_flag=1
	# is file readable?
	elif [ ! -r $dataFile ]; then
		echo "Datafile ($datafile) is unreadable. Exiting."
		echo "Url used: $kernelURL"
		exit 1
	fi

	exec 3<&0
	exec 0<$dataFile

	while read line; do
		lhs="${line%:*}"
		rhs="${line#*:}"

		if [[ $lhs == "LatestStable" ]]; then
			orig_stable=$rhs
		elif [[ $lhs == "LatestPrepatch" ]]; then
			orig_prepatch=$rhs
		elif [[ $lhs == "LatestSnapshot" ]]; then
			orig_snapshot=$rhs
		elif [[ $lhs == "LatestTwoFour" ]]; then
			orig_twofour=$rhs
		fi
	done

	# compare the file stored results to the downloaded ones
	alert=false

	if [ $first_run_flag == 0 ]; then
		if [[ $orig_stable != $stable ]]; then
			alert=true
			message="$message\nStable Kernel Version change: $orig_stable -> $stable"
		fi
		if [[ $orig_prepatch != $prepatch ]]; then
			alert=true
			message="$message\nPrepatch Kernel Version change: $orig_prepatch -> $prepatch"
		fi
		if [[ $orig_snapshot != $snapshot ]]; then
			alert=true
			message="$message\nSnapshot Kernel Version change: $orig_snapshot -> $snapshot"
		fi
		if [[ $orig_twofour != $twofour ]]; then
			alert=true
			message="$message\nStable Kernel Version change: $orig_twofour -> $twofour"
		fi
	fi

	exec 0<&3
}

readDownloadedKernelFile() {

	# did file download?
	if [ ! -e $kernelFile ]; then
		echo "Could not download http://www.kernel.org. Are you connected to the internet and is the URL still valid?"
		echo "Url used: $kernelURL"
		exit 1
	# is file readable?
	elif [ ! -r $kernelFile ]; then
		echo "Downloaded file from kernel.org ($kernelFile) is unreadable. Exiting."
		echo "Url used: $kernelURL"
		exit 1
	fi

	#strip quotes from file
	sed 's/\"//g' $kernelFile > $tempFile

	# open file
	exec 3<&0
	exec 0<$tempFile
	while read line
	do
		parseLine $line
	done
	exec 0<&3

	rm $tempFile
	rm $kernelFile
}

mailFunction() {

${mailCommand} -t -oi <<EOF
From: ${mailFrom}
To: ${mailTo}
Subject: ${subject}

`echo -e ${message}`

EOF
}


#download file
wget -q $kernelURL

# open the file and extract the data
readDownloadedKernelFile

# open the current data file and compare the extracted
loadKernelAlertDataFile

#email alert if necessary
if [[ $alert == true ]]; then
	echo -e "Alert: $message"

	# mail alert
	mailFunction
fi

#refresh data file
rm  $dataFile
echo -e "LatestStable:$stable\nLatestPrepatch:$prepatch\nLatestSnapshot:$snapshot\nLatestTwoFour:$twofour" > $dataFile
